@@ -28,6 +28,7 @@
 #include <gnome.h>
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
+#include <X11/Xlib.h>
 #include <X11/Xresource.h>
 #include <math.h>
 
@@ -36,7 +37,7 @@
 #include "activate-settings-daemon.h"
 
 #define IDIR GNOMECC_DATA_DIR "/pixmaps/"
-#define CONFIG_ROOT "/desktop/gnome/accesibility/keyboard"
+#define CONFIG_ROOT "/desktop/gnome/accessibility/keyboard"
 
 static struct {
 	char const * const checkbox;
@@ -80,11 +81,11 @@ static struct {
 	  CONFIG_ROOT "/bouncekeys_delay", FALSE },
 	{ "slowkeys_delay_slide",	"slowkeys_delay_spin",	    300,  10, 900,  10,
 	  CONFIG_ROOT "/slowkeys_delay", FALSE },
-	{ "mousekeys_max_speed_slide",	"mousekeys_max_speed_spin",   70,  10, 500,  10,
+	{ "mousekeys_max_speed_slide",	"mousekeys_max_speed_spin",   10,  1, 100,  5,
 	  CONFIG_ROOT "/mousekeys_max_speed", FALSE },
-	{ "mousekeys_accel_time_slide",	"mousekeys_accel_time_spin", 300,  10, 900,  10,
+	{ "mousekeys_accel_time_slide",	"mousekeys_accel_time_spin", 300,  10, 5000,  100,
 	  CONFIG_ROOT "/mousekeys_accel_time", FALSE },
-	{ "mousekeys_init_delay_slide",	"mousekeys_init_delay_spin", 200,  10, 500,  10,
+	{ "mousekeys_init_delay_slide",	"mousekeys_init_delay_spin", 300,  10, 5000,  10,
 	  CONFIG_ROOT "/mousekeys_init_delay", FALSE },
 	{ "timeout_slide",	"timeout_spin", 200,  10, 500,  10,
 	  CONFIG_ROOT "/timeout", TRUE },
@@ -235,9 +236,10 @@ setup_images (GladeXML *dialog, gboolean as_dialog)
 	int i = G_N_ELEMENTS (features);
 	while (i-- > 0)
 		if (features [i].image != NULL &&
-		    (as_dialog || !features [i].only_for_dialog))
+		    (as_dialog || !features [i].only_for_dialog)) {
 			gtk_image_set_from_file (GTK_IMAGE (WID (features [i].image)),
 				features [i].image_file);
+		}
 }
 
 static void
@@ -247,6 +249,7 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset, gboolean as_dialog)
 	GtkWidget *page = WID ("accessX_page");
 	GObject *label;
 
+
 	g_return_if_fail (content != NULL);
 	g_return_if_fail (page != NULL);
 
@@ -256,7 +259,7 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset, gboolean as_dialog)
 	setup_simple_toggles (dialog, changeset, as_dialog);
 
 	label = g_object_new (GTK_TYPE_CHECK_BUTTON,
-		"label",	 _("_Enable keyboard accesibility"),
+		"label",	 _("_Enable keyboard accessibility"),
 		"use_underline", TRUE,
 		/* init true so that if gconf is false toggle will fire */
 		"active",	 TRUE,
@@ -465,12 +468,18 @@ GtkWidget *
 setup_accessX_dialog (GConfChangeSet *changeset, gboolean as_dialog)
 {
 	GConfClient *client;
+#if 1
 	char const  *toplevel_name = as_dialog ? "accessX_dialog" : "accessX_page";
 	GladeXML    *dialog = glade_xml_new (GNOMECC_DATA_DIR
 		"/interfaces/gnome-accessibility-keyboard-properties.glade2",
 		toplevel_name, NULL);
 	GtkWidget   *toplevel = WID (toplevel_name);
-
+#else
+	char const  *toplevel_name = "key_access_dialog";
+	GladeXML    *dialog = glade_xml_new ("access-foo.glade",
+					     toplevel_name, NULL);
+	GtkWidget   *toplevel = WID (toplevel_name);
+#endif
 	client = gconf_client_get_default ();
 	gconf_client_add_dir (client, CONFIG_ROOT, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
@@ -479,7 +488,7 @@ setup_accessX_dialog (GConfChangeSet *changeset, gboolean as_dialog)
 	if (as_dialog) {
 		GtkWidget *load_cde = WID ("load_CDE_file");
 		g_signal_connect (G_OBJECT (load_cde),
-			"activate",
+			"clicked",
 			G_CALLBACK (cb_load_CDE_file), toplevel);
 	}
 	return toplevel;
