@@ -31,6 +31,8 @@
 #include "gconf-property-editor.h"
 #include "gconf-property-editor-marshal.h"
 
+#include "preview-file-selection.h"
+
 enum {
 	VALUE_CHANGED,
 	LAST_SIGNAL
@@ -43,9 +45,13 @@ enum {
 	PROP_CHANGESET,
 	PROP_CONV_TO_WIDGET_CB,
 	PROP_CONV_FROM_WIDGET_CB,
-	PROP_UI_CONTROL
+	PROP_UI_CONTROL,
+	PROP_DATA,
+	PROP_DATA_FREE_CB
 };
 
+typedef void (*GConfPropertyEditorDataFreeCb) (gpointer data);
+
 struct _GConfPropertyEditorPrivate 
 {
 	gchar                   *key;
@@ -56,8 +62,20 @@ struct _GConfPropertyEditorPrivate
 	GConfPEditorValueConvFn  conv_from_widget_cb;
 	GConfClientNotifyFunc    callback;
 	gboolean                 inited;
+
+	gpointer 		      data;
+	GConfPropertyEditorDataFreeCb data_free_cb;
 };
 
+typedef struct
+{
+	GType			 enum_type;
+	GConfPEditorGetValueFn   enum_val_true_fn;
+	gpointer		 enum_val_true_fn_data;
+	guint			 enum_val_false;
+	gboolean		 use_nick;
+} GConfPropertyEditorEnumData;
+
 static guint peditor_signals[LAST_SIGNAL];
 
 static GObjectClass *parent_class;
@@ -83,7 +101,9 @@ static GObject *gconf_peditor_new             (gchar                 *key,
 					       GConfChangeSet        *changeset,
 					       GObject               *ui_control,
 					       const gchar           *first_prop_name,
-					       va_list                var_args);
+					       va_list                var_args,
+					       const gchar	     *first_custom,
+					       ...);
 
 GType
 gconf_property_editor_get_type (void)
@@ -113,13 +133,20 @@ gconf_property_editor_get_type (void)
 	return gconf_property_editor_type;
 }
 
+static GConfValue* 
+gconf_property_editor_conv_default (GConfPropertyEditor *peditor,
+				    const GConfValue *value)
+{
+	return gconf_value_copy (value);
+}
+
 static void
 gconf_property_editor_init (GConfPropertyEditor      *gconf_property_editor,
 			    GConfPropertyEditorClass *class)
 {
 	gconf_property_editor->p = g_new0 (GConfPropertyEditorPrivate, 1);
-	gconf_property_editor->p->conv_to_widget_cb = gconf_value_copy;
-	gconf_property_editor->p->conv_from_widget_cb = gconf_value_copy;
+	gconf_property_editor->p->conv_to_widget_cb = gconf_property_editor_conv_default;
+	gconf_property_editor->p->conv_from_widget_cb = gconf_property_editor_conv_default;
 	gconf_property_editor->p->inited = FALSE;
 }
 
@@ -186,6 +213,20 @@ gconf_property_editor_class_init (GConfPropertyEditorClass *class)
 			      (GSignalCMarshaller) gconf_property_editor_marshal_VOID__STRING_POINTER,
 			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
 
+	g_object_class_install_property
+		(object_class, PROP_DATA,
+		 g_param_spec_pointer ("data",
+				       _("Property editor object data"),
+				       _("Custom data required by the specific property editor"),
+				       G_PARAM_WRITABLE));
+
+	g_object_class_install_property
+		(object_class, PROP_DATA_FREE_CB,
+		 g_param_spec_pointer ("data-free-cb",
+				       _("Property editor data freeing callback"),
+				       _("Callback to be issued when property editor object data is to be freed"),
+				       G_PARAM_WRITABLE));
+
 	parent_class = G_OBJECT_CLASS
 		(g_type_class_ref (G_TYPE_OBJECT));
 }
@@ -236,7 +277,12 @@ gconf_property_editor_set_prop (GObject      *object,
 		peditor->p->ui_control = g_value_get_object (value);
 		g_object_weak_ref (peditor->p->ui_control, (GWeakNotify) g_object_unref, object);
 		break;
-
+	case PROP_DATA:
+		peditor->p->data = g_value_get_pointer (value);
+		break;
+	case PROP_DATA_FREE_CB:
+		peditor->p->data_free_cb = g_value_get_pointer (value);
+		break;
 	default:
 		g_warning ("Bad argument set");
 		break;
@@ -281,6 +327,9 @@ gconf_property_editor_finalize (GObject *object)
 
 	gconf_property_editor = GCONF_PROPERTY_EDITOR (object);
 
+	if (gconf_property_editor->p->data_free_cb)
+		gconf_property_editor->p->data_free_cb (gconf_property_editor->p->data);
+	
 	g_free (gconf_property_editor->p);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -292,7 +341,9 @@ gconf_peditor_new (gchar                 *key,
 		   GConfChangeSet        *changeset,
 		   GObject               *ui_control,
 		   const gchar           *first_prop_name,
-		   va_list                var_args) 
+		   va_list                var_args,
+		   const gchar		 *first_custom,
+		   ...) 
 {
 	GObject     *obj;
 	GConfClient *client;
@@ -310,6 +361,14 @@ gconf_peditor_new (gchar                 *key,
 
 	g_object_set_valist (obj, first_prop_name, var_args);
 
+	if (first_custom)
+	{
+		va_list custom_args;
+		va_start (custom_args, first_custom);
+		g_object_set_valist (obj, first_custom, custom_args);
+		va_end (custom_args);
+	}
+
 	client = gconf_client_get_default ();
 	gconf_entry = gconf_client_get_entry (client, GCONF_PROPERTY_EDITOR (obj)->p->key, NULL, TRUE, NULL);
 	GCONF_PROPERTY_EDITOR (obj)->p->callback (client, 0, gconf_entry, obj);
@@ -349,7 +408,7 @@ peditor_boolean_value_changed (GConfClient         *client,
 	value = gconf_entry_get_value (entry);
 
 	if (value != NULL) {
-		value_wid = peditor->p->conv_to_widget_cb (value);
+		value_wid = peditor->p->conv_to_widget_cb (peditor, value);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (peditor->p->ui_control), gconf_value_get_bool (value_wid));
 		gconf_value_free (value_wid);
 	}
@@ -364,7 +423,7 @@ peditor_boolean_widget_changed (GConfPropertyEditor *peditor,
 	if (!peditor->p->inited) return;
 	value_wid = gconf_value_new (GCONF_VALUE_BOOL);
 	gconf_value_set_bool (value_wid, gtk_toggle_button_get_active (tb));
-	value = peditor->p->conv_from_widget_cb (value_wid);
+	value = peditor->p->conv_from_widget_cb (peditor, value_wid);
 	peditor_set_gconf_value (peditor, peditor->p->key, value);
 	g_signal_emit (peditor, peditor_signals[VALUE_CHANGED], 0, peditor->p->key, value);
 	gconf_value_free (value_wid);
@@ -393,7 +452,8 @@ gconf_peditor_new_boolean (GConfChangeSet *changeset,
 		 changeset,
 		 G_OBJECT (checkbox),
 		 first_property_name,
-		 var_args);
+		 var_args,
+		 NULL);
 
 	va_end (var_args);
 
@@ -418,7 +478,7 @@ peditor_string_value_changed (GConfClient         *client,
 	value = gconf_entry_get_value (entry);
 
 	if (value != NULL) {
-		value_wid = peditor->p->conv_to_widget_cb (value);
+		value_wid = peditor->p->conv_to_widget_cb (peditor, value);
 		entry_current_text = gtk_entry_get_text (GTK_ENTRY (peditor->p->ui_control));
 		if (strcmp (entry_current_text, gconf_value_get_string (value)) != 0) {
 		  gtk_entry_set_text (GTK_ENTRY (peditor->p->ui_control), gconf_value_get_string (value_wid));
@@ -438,7 +498,7 @@ peditor_string_widget_changed (GConfPropertyEditor *peditor,
 	value_wid = gconf_value_new (GCONF_VALUE_STRING);
 
 	gconf_value_set_string (value_wid, gtk_entry_get_text (entry));
-	value = peditor->p->conv_from_widget_cb (value_wid);
+	value = peditor->p->conv_from_widget_cb (peditor, value_wid);
 
 	peditor_set_gconf_value (peditor, peditor->p->key, value);
 
@@ -462,7 +522,7 @@ gconf_peditor_new_string_valist (GConfChangeSet *changeset,
 		 changeset,
 		 G_OBJECT (entry),
 		 first_property_name,
-		 var_args);
+		 var_args, NULL);
 
 	g_signal_connect_swapped (G_OBJECT (entry), "changed",
 				  (GCallback) peditor_string_widget_changed, peditor);
@@ -536,7 +596,7 @@ peditor_color_value_changed (GConfClient         *client,
 	value = gconf_entry_get_value (entry);
 
 	if (value != NULL) {
-		value_wid = peditor->p->conv_to_widget_cb (value);
+		value_wid = peditor->p->conv_to_widget_cb (peditor, value);
 		gdk_color_parse (gconf_value_get_string (value_wid), &color);
 		gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (peditor->p->ui_control), color.red, color.green, color.blue, 65535);
 		gconf_value_free (value_wid);
@@ -561,7 +621,7 @@ peditor_color_widget_changed (GConfPropertyEditor *peditor,
 	gconf_value_set_string (value_wid, str);
 	g_free (str);
 
-	value = peditor->p->conv_from_widget_cb (value_wid);
+	value = peditor->p->conv_from_widget_cb (peditor, value_wid);
 
 	peditor_set_gconf_value (peditor, peditor->p->key, value);
 	g_signal_emit (peditor, peditor_signals[VALUE_CHANGED], 0, peditor->p->key, value);
@@ -592,7 +652,7 @@ gconf_peditor_new_color (GConfChangeSet *changeset,
 		 changeset,
 		 G_OBJECT (cp),
 		 first_property_name,
-		 var_args);
+		 var_args, NULL);
 
 	va_end (var_args);
 
@@ -602,6 +662,97 @@ gconf_peditor_new_color (GConfChangeSet *changeset,
 	return peditor;
 }
 
+static int
+peditor_enum_int_from_string (GType type, const gchar *str, gboolean *use_nick)
+{
+	GEnumClass *klass;
+	GEnumValue *val;
+	int ret = -1;
+
+	if (use_nick)
+		*use_nick = FALSE;
+	
+	klass = g_type_class_ref (type);
+	val = g_enum_get_value_by_name (klass, str);
+	if (!val)
+	{
+		val = g_enum_get_value_by_nick (klass, str);
+		if (use_nick)
+			*use_nick = TRUE;
+	}
+	if (val)
+		ret = val->value;
+
+	g_type_class_unref (klass);
+
+	return ret;
+}
+
+static gchar*
+peditor_enum_string_from_int (GType type, const int index, gboolean use_nick)
+{
+	GEnumClass *klass;
+	GEnumValue *val;
+	gchar *ret = NULL;
+
+	klass = g_type_class_ref (type);
+	val = g_enum_get_value (klass, index);
+	if (val)
+	{
+		if (val->value_nick && use_nick)
+			ret = g_strdup (val->value_nick);
+		else
+			ret = g_strdup (val->value_name);
+	}
+	
+	g_type_class_unref (klass);
+
+	return ret;
+}
+
+static GConfValue*
+peditor_enum_conv_to_widget (GConfPropertyEditor *peditor,
+			     const GConfValue *value)
+{
+	GConfValue *ret;
+	GConfPropertyEditorEnumData *data = peditor->p->data;
+	int index;
+	
+	if (value->type == GCONF_VALUE_INT)
+		return gconf_value_copy (value);
+	
+	ret = gconf_value_new (GCONF_VALUE_INT);
+	
+	index = peditor_enum_int_from_string (data->enum_type,
+		    		    	      gconf_value_get_string (value),
+					      &data->use_nick);
+	
+	gconf_value_set_int (ret, index);
+
+	return ret;
+}
+
+static GConfValue*
+peditor_enum_conv_from_widget (GConfPropertyEditor *peditor,
+			       const GConfValue *value)
+{
+	GConfValue *ret;
+	GConfPropertyEditorEnumData *data = peditor->p->data;
+	gchar *str;
+
+	if (value->type == GCONF_VALUE_STRING)
+		return gconf_value_copy (value);
+
+	ret = gconf_value_new (GCONF_VALUE_STRING);
+	str = peditor_enum_string_from_int (data->enum_type,
+					    gconf_value_get_int (value),
+					    data->use_nick);
+	gconf_value_set_string (ret, str);
+	g_free (str);
+
+	return ret;
+}
+
 static void
 peditor_select_menu_value_changed (GConfClient         *client,
 				   guint                cnxn_id,
@@ -616,7 +767,7 @@ peditor_select_menu_value_changed (GConfClient         *client,
 	value = gconf_entry_get_value (entry);
 
 	if (value != NULL) {
-		value_wid = peditor->p->conv_to_widget_cb (value);
+		value_wid = peditor->p->conv_to_widget_cb (peditor, value);
 		gtk_option_menu_set_history (GTK_OPTION_MENU (peditor->p->ui_control), gconf_value_get_int (value_wid));
 		gconf_value_free (value_wid);
 	}
@@ -631,7 +782,7 @@ peditor_select_menu_widget_changed (GConfPropertyEditor *peditor,
 	if (!peditor->p->inited) return;
 	value_wid = gconf_value_new (GCONF_VALUE_INT);
 	gconf_value_set_int (value_wid, gtk_option_menu_get_history (option_menu));
-	value = peditor->p->conv_from_widget_cb (value_wid);
+	value = peditor->p->conv_from_widget_cb (peditor, value_wid);
 	peditor_set_gconf_value (peditor, peditor->p->key, value);
 	g_signal_emit (peditor, peditor_signals[VALUE_CHANGED], 0, peditor->p->key, value);
 	gconf_value_free (value_wid);
@@ -660,7 +811,7 @@ gconf_peditor_new_select_menu (GConfChangeSet *changeset,
 		 changeset,
 		 G_OBJECT (option_menu),
 		 first_property_name,
-		 var_args);
+		 var_args, NULL);
 
 	va_end (var_args);
 
@@ -670,13 +821,62 @@ gconf_peditor_new_select_menu (GConfChangeSet *changeset,
 	return peditor;
 }
 
+GObject *
+gconf_peditor_new_select_menu_with_enum	(GConfChangeSet *changeset,
+					 gchar 	        *key,
+					 GtkWidget      *option_menu,
+					 GType          enum_type,
+					 gchar          *first_property_name,
+					 ...)
+{
+	GConfPropertyEditor *peditor;
+	GConfPropertyEditorEnumData *data;
+	va_list var_args;
+
+	g_return_val_if_fail (key != NULL, NULL);
+	g_return_val_if_fail (option_menu != NULL, NULL);
+	g_return_val_if_fail (GTK_IS_OPTION_MENU (option_menu), NULL);
+	g_return_val_if_fail (enum_type != G_TYPE_NONE, NULL);
+
+	data = g_new0 (GConfPropertyEditorEnumData, 1);
+	data->enum_type = enum_type;
+
+	va_start (var_args, first_property_name);
+
+	peditor = GCONF_PROPERTY_EDITOR (
+		gconf_peditor_new
+		(key,
+		 (GConfClientNotifyFunc) peditor_select_menu_value_changed,
+		 changeset,
+		 G_OBJECT (option_menu),
+		 first_property_name,
+		 var_args,
+		 "conv-to-widget-cb",
+		 peditor_enum_conv_to_widget,
+		 "conv-from-widget-cb",
+		 peditor_enum_conv_from_widget,
+		 "data",
+		 data,
+		 "data-free-cb",
+		 g_free,
+		 NULL
+		 ));
+
+	va_end (var_args);
+	
+	g_signal_connect_swapped (G_OBJECT (option_menu), "changed",
+				  (GCallback) peditor_select_menu_widget_changed, peditor);
+
+	return G_OBJECT (peditor);
+}
+
 static void
 peditor_select_radio_value_changed (GConfClient         *client,
 				    guint                cnxn_id,
 				    GConfEntry          *entry,
 				    GConfPropertyEditor *peditor) 
 {
-	GSList *group;
+	GSList *group, *link;
 	GConfValue *value, *value_wid;
 
 	if (peditor->p->changeset != NULL)
@@ -685,12 +885,14 @@ peditor_select_radio_value_changed (GConfClient         *client,
 	value = gconf_entry_get_value (entry);
 
 	if (value != NULL) {
-		value_wid = peditor->p->conv_to_widget_cb (value);
-		group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (peditor->p->ui_control));
-		group = g_slist_nth (group, gconf_value_get_int (value_wid));
-		if (group && group->data)
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (group->data), TRUE);
+		value_wid = peditor->p->conv_to_widget_cb (peditor, value);
+		group = g_slist_copy (gtk_radio_button_get_group (GTK_RADIO_BUTTON (peditor->p->ui_control)));
+		group = g_slist_reverse (group);
+		link = g_slist_nth (group, gconf_value_get_int (value_wid));
+		if (link && link->data)
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (link->data), TRUE);
 		gconf_value_free (value_wid);
+		g_slist_free (group);
 	}
 }
 
@@ -705,15 +907,18 @@ peditor_select_radio_widget_changed (GConfPropertyEditor *peditor,
 	if (!tb->active) return;
 
 	value_wid = gconf_value_new (GCONF_VALUE_INT);
-	group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (peditor->p->ui_control));
+	group = g_slist_copy (gtk_radio_button_get_group (GTK_RADIO_BUTTON (peditor->p->ui_control)));
+	group = g_slist_reverse (group);
+	
 	gconf_value_set_int (value_wid, g_slist_index (group, tb));
-	value = peditor->p->conv_from_widget_cb (value_wid);
+	value = peditor->p->conv_from_widget_cb (peditor, value_wid);
 
 	peditor_set_gconf_value (peditor, peditor->p->key, value);
 	g_signal_emit (peditor, peditor_signals[VALUE_CHANGED], 0, peditor->p->key, value);
 
 	gconf_value_free (value_wid);
 	gconf_value_free (value);
+	g_slist_free (group);
 }
 
 GObject *
@@ -743,7 +948,7 @@ gconf_peditor_new_select_radio (GConfChangeSet *changeset,
 		 changeset,
 		 G_OBJECT (first_button),
 		 first_property_name,
-		 var_args);
+		 var_args, NULL);
 
 	va_end (var_args);
 
@@ -768,7 +973,10 @@ peditor_numeric_range_value_changed (GConfClient         *client,
 	value = gconf_entry_get_value (entry);
 
 	if (value != NULL) {
-		value_wid = peditor->p->conv_to_widget_cb (value);
+		value_wid = peditor->p->conv_to_widget_cb (peditor, value);
+
+		g_return_if_fail  (value_wid->type == GCONF_VALUE_FLOAT);
+
 		gtk_adjustment_set_value (GTK_ADJUSTMENT (peditor->p->ui_control), gconf_value_get_float (value_wid));
 		gconf_value_free (value_wid);
 	}
@@ -783,7 +991,7 @@ peditor_numeric_range_widget_changed (GConfPropertyEditor *peditor,
 	if (!peditor->p->inited) return;
 	value_wid = gconf_value_new (GCONF_VALUE_FLOAT);
 	gconf_value_set_float (value_wid, gtk_adjustment_get_value (adjustment));
-	value = peditor->p->conv_from_widget_cb (value_wid);
+	value = peditor->p->conv_from_widget_cb (peditor, value_wid);
 	peditor_set_gconf_value (peditor, peditor->p->key, value);
 	g_signal_emit (peditor, peditor_signals[VALUE_CHANGED], 0, peditor->p->key, value);
 	gconf_value_free (value_wid);
@@ -815,7 +1023,7 @@ gconf_peditor_new_numeric_range (GConfChangeSet *changeset,
 		 changeset,
 		 G_OBJECT (adjustment),
 		 first_property_name,
-		 var_args);
+		 var_args, NULL);
 
 	va_end (var_args);
 
@@ -825,13 +1033,26 @@ gconf_peditor_new_numeric_range (GConfChangeSet *changeset,
 	return peditor;
 }
 
+static gboolean
+guard_get_bool (GConfPropertyEditor *peditor, const GConfValue *value)
+{
+	if (value->type == GCONF_VALUE_BOOL)
+		return gconf_value_get_bool (value);
+	else
+	{
+		GConfPropertyEditorEnumData *data = peditor->p->data;
+		int index = peditor_enum_int_from_string (data->enum_type, gconf_value_get_string (value), &data->use_nick);
+		return (index != data->enum_val_false);
+	}
+}
+
 static void
 guard_value_changed (GConfPropertyEditor *peditor,
 		     const gchar         *key,
 		     const GConfValue    *value,
 		     GtkWidget           *widget) 
 {
-	gtk_widget_set_sensitive (widget, gconf_value_get_bool (value));
+	gtk_widget_set_sensitive (widget, guard_get_bool (peditor, value));
 }
 
 void
@@ -839,6 +1060,7 @@ gconf_peditor_widget_set_guard (GConfPropertyEditor *peditor,
 				GtkWidget           *widget)
 {
 	GConfClient *client;
+	GConfValue *value;
 
 	g_return_if_fail (peditor != NULL);
 	g_return_if_fail (IS_GCONF_PROPERTY_EDITOR (peditor));
@@ -846,7 +1068,15 @@ gconf_peditor_widget_set_guard (GConfPropertyEditor *peditor,
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 
 	client = gconf_client_get_default ();
-	gtk_widget_set_sensitive (widget, gconf_client_get_bool (client, peditor->p->key, NULL));
+	
+	value = gconf_client_get (client, peditor->p->key, NULL);
+
+	if (value) {
+		gtk_widget_set_sensitive (widget, guard_get_bool (peditor, value));
+        	gconf_value_free (value);
+	} else {
+                g_warning ("NULL GConf value: %s: possibly incomplete setup", peditor->p->key);
+	}
 
 	g_signal_connect (G_OBJECT (peditor), "value-changed", (GCallback) guard_value_changed, widget);
 }
@@ -872,20 +1102,71 @@ gconf_value_float_to_int (const GConfValue *value)
 }
 
 static gint
-peditor_font_get_size (const gchar *font_name)
+peditor_font_get_size (const gchar *font_str)
 {
 	PangoFontDescription *desc;
 	int size;
 	
-	g_return_val_if_fail (font_name != NULL, -1);
+	g_return_val_if_fail (font_str != NULL, -1);
 	
-	desc = pango_font_description_from_string (font_name);
+	desc = pango_font_description_from_string (font_str);
 	size = pango_font_description_get_size (desc);
 	pango_font_description_free (desc);
 
 	return size / PANGO_SCALE;
 }
 
+static gchar* 
+peditor_font_get_name (const gchar *font_str)
+{
+	PangoFontDescription *desc;
+	gchar *name;
+
+	g_return_val_if_fail (font_str != NULL, NULL);
+	
+	desc = pango_font_description_from_string (font_str);
+	pango_font_description_unset_fields (desc, PANGO_FONT_MASK_SIZE);
+	name = pango_font_description_to_string (desc);
+	pango_font_description_free (desc);
+
+	return name;
+}
+
+static void
+peditor_font_merge_setting (GnomeFontPicker *font_picker, GConfValue *value, GConfPEditorFontType font_type)
+{
+	PangoFontDescription *desc;
+	gchar *font_str;
+	
+	g_return_if_fail (GNOME_IS_FONT_PICKER (font_picker));
+	g_return_if_fail (value != NULL);
+	
+	
+	desc = pango_font_description_from_string (gnome_font_picker_get_font_name (font_picker));
+	
+	if (font_type == PEDITOR_FONT_SIZE)
+	{
+		pango_font_description_set_size (desc, gconf_value_get_int (value) * PANGO_SCALE);
+	}
+	else
+	{
+		PangoFontDescription *new_desc;
+		new_desc = pango_font_description_from_string (gconf_value_get_string (value));
+		if (font_type == PEDITOR_FONT_NAME)
+			pango_font_description_set_size (new_desc, pango_font_description_get_size (desc));
+		pango_font_description_free (desc);
+		desc = new_desc;
+	}
+
+	font_str = pango_font_description_to_string (desc);
+	g_object_set (G_OBJECT (font_picker),
+		      "font_name", font_str,
+		      "label-font-size", pango_font_description_get_size (desc) / PANGO_SCALE,
+		      NULL);
+	g_free (font_str);
+	pango_font_description_free (desc);
+}
+
 static void
 peditor_font_value_changed (GConfClient         *client,
 			     guint                cnxn_id,
@@ -893,41 +1174,59 @@ peditor_font_value_changed (GConfClient         *client,
 			     GConfPropertyEditor *peditor) 
 {
 	GConfValue *value, *value_wid;
+	GConfPEditorFontType font_type;
 
 	if (peditor->p->changeset != NULL)
 		gconf_change_set_remove (peditor->p->changeset, peditor->p->key);
 
+	font_type = GPOINTER_TO_UINT (peditor->p->data);
 	value = gconf_entry_get_value (entry);
 
 	if (value != NULL) {
-		const gchar *font_name;
-
-		value_wid = peditor->p->conv_to_widget_cb (value);
-		font_name = gconf_value_get_string (value_wid);
-		g_object_set (G_OBJECT (peditor->p->ui_control),
-			      "font_name", font_name,
-			      "label-font-size", peditor_font_get_size (font_name),
-			      NULL);
+		value_wid = peditor->p->conv_to_widget_cb (peditor, value);
+		peditor_font_merge_setting (GNOME_FONT_PICKER (peditor->p->ui_control), value_wid, font_type);
 		gconf_value_free (value_wid);
 	}
 }
 
 static void
 peditor_font_widget_changed (GConfPropertyEditor *peditor,
-			      gchar              *font_name,
+			      gchar              *font_str,
 			      GnomeFontPicker    *font_picker)
 {
-	GConfValue *value, *value_wid;
+	GConfValue *value, *value_wid = NULL;
+	GConfPEditorFontType font_type;
+	gchar *font_name;
+	int font_size;
 
 	if (!peditor->p->inited) return;
 
+	font_type = GPOINTER_TO_UINT (peditor->p->data);
+
+	font_size = peditor_font_get_size (font_str);
 	g_object_set (G_OBJECT (peditor->p->ui_control),
-		      "label-font-size", peditor_font_get_size (font_name),
+		      "label-font-size", font_size,
 		      NULL);
 	
-	value_wid = gconf_value_new (GCONF_VALUE_STRING);
-	gconf_value_set_string (value_wid, font_name);
-	value = peditor->p->conv_from_widget_cb (value_wid);
+	switch (font_type)
+	{
+		case PEDITOR_FONT_NAME:
+			value_wid = gconf_value_new (GCONF_VALUE_STRING);
+			font_name = peditor_font_get_name (font_str);
+			gconf_value_set_string (value_wid, font_name);
+			g_free (font_name);
+			break;
+		case PEDITOR_FONT_SIZE:
+			value_wid = gconf_value_new (GCONF_VALUE_INT);
+			gconf_value_set_int (value_wid, font_size);
+			break;
+		case PEDITOR_FONT_COMBINED:
+			value_wid = gconf_value_new (GCONF_VALUE_STRING);
+			gconf_value_set_string (value_wid, font_str);
+			break;
+	}
+	
+	value = peditor->p->conv_from_widget_cb (peditor, value_wid);
 
 	peditor_set_gconf_value (peditor, peditor->p->key, value);
 	g_signal_emit (peditor, peditor_signals[VALUE_CHANGED], 0, peditor->p->key, value);
@@ -940,6 +1239,7 @@ GObject *
 gconf_peditor_new_font (GConfChangeSet *changeset,
 			gchar *key,
 			GtkWidget *font_picker,
+			GConfPEditorFontType font_type,
 			gchar *first_property_name,
 			...)
 {
@@ -958,7 +1258,9 @@ gconf_peditor_new_font (GConfChangeSet *changeset,
 		 changeset,
 		 G_OBJECT (font_picker),
 		 first_property_name,
-		 var_args);
+		 var_args,
+		 "data", GUINT_TO_POINTER (font_type),
+		 NULL);
 
 	va_end (var_args);
 
@@ -968,3 +1270,359 @@ gconf_peditor_new_font (GConfChangeSet *changeset,
 	return peditor;
 }
 
+static GConfValue*
+peditor_enum_toggle_conv_to_widget (GConfPropertyEditor *peditor,
+				    const GConfValue *value)
+{
+	GConfValue *ret;
+	GConfPropertyEditorEnumData *data = peditor->p->data;
+	int index;
+	
+	if (value->type == GCONF_VALUE_BOOL)
+		return gconf_value_copy (value);
+	
+	ret = gconf_value_new (GCONF_VALUE_BOOL);
+	
+	index = peditor_enum_int_from_string (data->enum_type,
+		    		    	      gconf_value_get_string (value),
+					      &data->use_nick);
+	gconf_value_set_bool (ret, (index != data->enum_val_false));
+
+	return ret;
+}
+
+static GConfValue*
+peditor_enum_toggle_conv_from_widget (GConfPropertyEditor *peditor,
+				      const GConfValue *value)
+{
+	GConfValue *ret;
+	GConfPropertyEditorEnumData *data = peditor->p->data;
+	gchar *str;
+	int index;
+
+	if (value->type == GCONF_VALUE_STRING)
+		return gconf_value_copy (value);
+
+	ret = gconf_value_new (GCONF_VALUE_STRING);
+	if (gconf_value_get_bool (value))
+		index = data->enum_val_true_fn (peditor, data->enum_val_true_fn_data);
+	else
+		index = data->enum_val_false;
+
+	str = peditor_enum_string_from_int (data->enum_type, index, data->use_nick);
+	gconf_value_set_string (ret, str);
+	g_free (str);
+
+	return ret;
+}
+
+GObject *
+gconf_peditor_new_enum_toggle  (GConfChangeSet 	 *changeset,
+				gchar			 *key,
+				GtkWidget		 *checkbox,
+				GType			 enum_type,
+				GConfPEditorGetValueFn   val_true_fn,
+				guint			 val_false,
+				gpointer		 data,
+				gchar 			 *first_property_name,
+				...)
+{
+	GConfPropertyEditor *peditor;
+	GConfPropertyEditorEnumData *enum_data;
+	va_list var_args;
+
+	g_return_val_if_fail (key != NULL, NULL);
+	g_return_val_if_fail (checkbox != NULL, NULL);
+	g_return_val_if_fail (GTK_IS_TOGGLE_BUTTON (checkbox), NULL);
+
+	enum_data = g_new0 (GConfPropertyEditorEnumData, 1);
+	enum_data->enum_type = enum_type;
+	enum_data->enum_val_true_fn = val_true_fn;
+	enum_data->enum_val_true_fn_data = data;
+	enum_data->enum_val_false = val_false;
+
+	va_start (var_args, first_property_name);
+
+	peditor = GCONF_PROPERTY_EDITOR (
+		gconf_peditor_new
+		(key,
+		 (GConfClientNotifyFunc) peditor_boolean_value_changed,
+		 changeset,
+		 G_OBJECT (checkbox),
+		 first_property_name,
+		 var_args,
+		 "conv-to-widget-cb",
+		 peditor_enum_toggle_conv_to_widget,
+		 "conv-from-widget-cb",
+		 peditor_enum_toggle_conv_from_widget,
+		 "data",
+		 enum_data,
+		 "data-free-cb",
+		 g_free,
+		 NULL));
+
+	va_end (var_args);
+
+	g_signal_connect_swapped (G_OBJECT (checkbox), "toggled",
+				  (GCallback) peditor_boolean_widget_changed, peditor);
+
+	return G_OBJECT (peditor);
+}
+
+gboolean
+peditor_image_set_filename (GConfPropertyEditor *peditor, const gchar *filename)
+{
+	GdkPixbuf *pixbuf = NULL;
+	GdkPixbuf *scaled;
+	GtkImage *image = NULL;
+	const int scale = 100;
+	gchar *message = NULL;
+	GList *l;
+	
+	if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+	{
+		message = g_strdup_printf (_("Couldn't find the file '%s'.\n\nPlease make "
+					      "sure it exists and try again, " 
+					      "or choose a different background picture."),
+					   filename);
+
+	}
+	else if (!(pixbuf = gdk_pixbuf_new_from_file (filename, NULL)))
+	{
+		message = g_strdup_printf (_("I don't know how to open the file '%s'.\n"
+					     "Perhaps it's "
+					     "a kind of picture that is not yet supported.\n\n"
+					     "Please select a different picture instead."),
+					   filename);
+	}
+
+	if (message)
+	{
+		GtkWidget *box;
+
+		box = gtk_message_dialog_new (NULL,
+					      GTK_DIALOG_MODAL,
+					      GTK_MESSAGE_ERROR,
+					      GTK_BUTTONS_OK,
+					      message);
+		gtk_dialog_run (GTK_DIALOG (box));
+		gtk_widget_destroy (box);
+		g_free (message);
+
+		return FALSE;
+	}
+
+	scaled = preview_file_selection_intelligent_scale (pixbuf,
+							   scale);
+
+	if (GTK_IS_IMAGE (GTK_BIN (peditor->p->ui_control)->child))
+		image = GTK_IMAGE (GTK_BIN (peditor->p->ui_control)->child);
+	else
+	{
+		for (l = gtk_container_get_children (GTK_CONTAINER (GTK_BIN (peditor->p->ui_control)->child)); l != NULL; l = l->next)
+		{	
+			if (GTK_IS_IMAGE (l->data))
+				image = GTK_IMAGE (l->data);
+			else if (GTK_IS_LABEL (l->data))
+			{
+				gchar *base = g_path_get_basename (filename);
+				gtk_label_set_text (GTK_LABEL (l->data), base);
+				g_free (base);
+			}
+		}
+	}
+	gtk_image_set_from_pixbuf (image, scaled);
+	g_object_unref (G_OBJECT (pixbuf));
+	g_object_unref (G_OBJECT (scaled));
+
+	return TRUE;
+}
+
+void
+peditor_image_fsel_ok_cb (GtkFileSelection *fsel, gpointer data)
+{
+	GConfValue *value, *value_wid;
+	GConfPropertyEditor *peditor;
+	const gchar *filename;
+
+	peditor = g_object_get_data (G_OBJECT (fsel), "peditor");
+
+	if (!peditor->p->inited)
+		return;
+
+	filename = gtk_file_selection_get_filename (fsel);
+	if (!(filename && peditor_image_set_filename (peditor, filename)))
+		return;
+
+	value_wid = gconf_value_new (GCONF_VALUE_STRING);
+	gconf_value_set_string (value_wid, gtk_file_selection_get_filename (fsel));
+	value = peditor->p->conv_from_widget_cb (peditor, value_wid);
+
+	peditor_set_gconf_value (peditor, peditor->p->key, value);
+	g_signal_emit (peditor, peditor_signals[VALUE_CHANGED], 0, peditor->p->key, value);
+
+	gconf_value_free (value_wid);
+	gconf_value_free (value);
+	gtk_widget_destroy (GTK_WIDGET (fsel));
+}
+
+void
+peditor_image_clicked_cb (GConfPropertyEditor *peditor, GtkButton *button)
+{
+	GConfValue *value = NULL, *value_wid;
+	const gchar *filename;
+	GtkWidget *fsel;
+
+	fsel = preview_file_selection_new (_("Please select an image."), TRUE);
+	gtk_window_set_modal (GTK_WINDOW (fsel), TRUE);
+
+	/* need the current filename */
+	if (peditor->p->changeset)
+		gconf_change_set_check_value (peditor->p->changeset, peditor->p->key, &value);
+
+	if (value)
+	{
+		/* the one we got is not a copy */
+		value = gconf_value_copy (value);
+	}
+	else
+	{
+		GConfClient *client = gconf_client_get_default ();
+		value = gconf_client_get (client, peditor->p->key, NULL);
+	}
+	
+	value_wid = peditor->p->conv_to_widget_cb (peditor, value);
+	filename = gconf_value_get_string (value_wid);
+
+	if (filename && strcmp (filename, ""))
+		gtk_file_selection_set_filename (GTK_FILE_SELECTION (fsel), filename);
+
+	g_object_set_data (G_OBJECT (fsel), "peditor", peditor);
+
+	g_signal_connect_swapped  (G_OBJECT (GTK_FILE_SELECTION (fsel)->ok_button),
+				   "clicked",
+				   (GCallback) peditor_image_fsel_ok_cb,
+				   fsel);
+	
+	g_signal_connect_swapped  (G_OBJECT (GTK_FILE_SELECTION (fsel)->cancel_button),
+				   "clicked",
+				   (GCallback) gtk_widget_destroy,
+				   fsel);
+
+	if (gtk_grab_get_current ())
+		gtk_grab_add (fsel);
+	
+	gtk_widget_show (fsel);
+
+	gconf_value_free (value);
+	gconf_value_free (value_wid);
+}
+
+static void
+peditor_image_value_changed (GConfClient         *client,
+			     guint                cnxn_id,
+			     GConfEntry          *entry,
+			     GConfPropertyEditor *peditor) 
+{
+	GConfValue *value, *value_wid;
+
+	if (peditor->p->changeset != NULL)
+		gconf_change_set_remove (peditor->p->changeset, peditor->p->key);
+
+	value = gconf_entry_get_value (entry);
+
+	if (value != NULL) {
+		const gchar *filename;
+
+		value_wid = peditor->p->conv_to_widget_cb (peditor, value);
+		filename = gconf_value_get_string (value_wid);
+		peditor_image_set_filename (peditor, filename);
+		gconf_value_free (value_wid);
+	}
+}
+
+GObject *
+gconf_peditor_new_image (GConfChangeSet	  *changeset,
+			 gchar	          *key,
+			 GtkWidget	  *button,
+			 gchar		  *first_property_name,
+			 ...)
+{
+	GObject *peditor;
+	va_list var_args;
+
+	g_return_val_if_fail (key != NULL, NULL);
+	g_return_val_if_fail (button != NULL, NULL);
+	g_return_val_if_fail (GTK_IS_BUTTON (button), NULL);
+
+	va_start (var_args, first_property_name);
+	
+	peditor = gconf_peditor_new
+		(key,
+		 (GConfClientNotifyFunc) peditor_image_value_changed,
+		 changeset,
+		 G_OBJECT (button),
+		 first_property_name,
+		 var_args, NULL);
+
+	va_end (var_args);
+
+	g_signal_connect_swapped (G_OBJECT (button), "clicked",
+				  (GCallback) peditor_image_clicked_cb, peditor);
+
+	return peditor;
+}
+
+GObject *
+gconf_peditor_new_select_radio_with_enum (GConfChangeSet *changeset,
+					  gchar		 *key,
+					  GSList 	 *radio_group,
+					  GType 	 enum_type,
+					  gchar          *first_property_name,
+					  ...)
+{
+	GConfPropertyEditor *peditor;
+	GConfPropertyEditorEnumData *enum_data;
+	GtkRadioButton *first_button;
+	GSList *item;
+	va_list var_args;
+
+	g_return_val_if_fail (key != NULL, NULL);
+	g_return_val_if_fail (radio_group != NULL, NULL);
+	g_return_val_if_fail (radio_group->data != NULL, NULL);
+	g_return_val_if_fail (GTK_IS_RADIO_BUTTON (radio_group->data), NULL);
+
+	enum_data = g_new0 (GConfPropertyEditorEnumData, 1);
+	enum_data->enum_type = enum_type;
+
+	first_button = GTK_RADIO_BUTTON (radio_group->data);
+
+	va_start (var_args, first_property_name);
+
+	peditor = GCONF_PROPERTY_EDITOR (
+		gconf_peditor_new
+		(key,
+		 (GConfClientNotifyFunc) peditor_select_radio_value_changed,
+		 changeset,
+		 G_OBJECT (first_button),
+		 first_property_name,
+		 var_args,
+		 "conv-to-widget-cb",
+		 peditor_enum_conv_to_widget,
+		 "conv-from-widget-cb",
+		 peditor_enum_conv_from_widget,
+		 "data",
+		 enum_data,
+		 "data-free-cb",
+		 g_free,
+		 NULL));
+
+	va_end (var_args);
+
+	for (item = radio_group; item != NULL; item = item->next)
+		g_signal_connect_swapped (G_OBJECT (item->data), "toggled",
+					  (GCallback) peditor_select_radio_widget_changed, peditor);
+
+	return G_OBJECT (peditor);
+}
+
