@@ -176,6 +176,16 @@ remove_cb (GtkButton *button, GladeXML *dialog)
 	selection_changed_cb (selection, dialog);
 }
 
+static void
+cb_file_type_dialog_response (GtkDialog *dialog, gint response_id)
+{
+	if (response_id == GTK_RESPONSE_APPLY) {
+		model_entry_commit_dirty_list ();
+		model_entry_commit_delete_list ();
+	} else
+		gtk_main_quit ();
+}
+
 static GladeXML *
 create_dialog (void) 
 {
@@ -188,8 +198,6 @@ create_dialog (void)
 	GtkTreeViewColumn *column;
 	GtkTreeSelection  *selection;
 
-	gint               col_offset;
-
 	dialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/file-types-properties.glade", "main_dialog", NULL);
 
 	model = GTK_TREE_MODEL (mime_types_model_new (FALSE));
@@ -197,19 +205,23 @@ create_dialog (void)
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), model);
 
-	/* Icon column */
+	/* Icon/description column */
+	column = gtk_tree_view_column_new ();
+
 	renderer = gtk_cell_renderer_pixbuf_new ();
-	gtk_tree_view_insert_column_with_attributes
-		(GTK_TREE_VIEW (treeview), -1, NULL, renderer,
-		 "pixbuf", MODEL_COLUMN_ICON,
-		 NULL);
+	gtk_tree_view_column_pack_start (column, renderer, FALSE);
+	gtk_tree_view_column_set_attributes
+		(column, renderer, "pixbuf", MODEL_COLUMN_ICON, NULL);
 
 	/* Description column */
 	renderer = gtk_cell_renderer_text_new ();
-	col_offset = gtk_tree_view_insert_column_with_attributes
-		(GTK_TREE_VIEW (treeview), -1, _("Description"), renderer,
-		 "text", MODEL_COLUMN_DESCRIPTION,
-		 NULL);
+	gtk_tree_view_column_pack_start (column, renderer, FALSE);
+	gtk_tree_view_column_set_attributes
+		(column, renderer, "text", MODEL_COLUMN_DESCRIPTION, NULL);
+
+	gtk_tree_view_column_set_title (column, _("Description"));
+	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (treeview), column);
 
 	/* Extensions column */
 	renderer = gtk_cell_renderer_text_new ();
@@ -218,9 +230,6 @@ create_dialog (void)
 		 "text", MODEL_COLUMN_FILE_EXT,
 		 NULL);
 
-	column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), col_offset - 1);
-	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (treeview), column);
-
 	gtk_widget_set_sensitive (WID ("edit_button"), FALSE);
 	gtk_widget_set_sensitive (WID ("remove_button"), FALSE);
 
@@ -235,14 +244,11 @@ create_dialog (void)
 
 	g_signal_connect (G_OBJECT (WID ("mime_types_tree")), "row-activated", (GCallback) row_activated_cb, dialog);
 
-	return dialog;
-}
+	g_signal_connect (G_OBJECT (WID ("main_dialog")),
+		"response",
+		G_CALLBACK (cb_file_type_dialog_response), NULL);
 
-static void
-apply_cb (void) 
-{
-	model_entry_commit_dirty_list ();
-	model_entry_commit_delete_list ();
+	return dialog;
 }
 
 static void
@@ -254,18 +260,18 @@ dialog_done_cb (MimeEditDialog *dialog, gboolean done, MimeTypeInfo *info)
 	gtk_main_quit ();
 }
 
+
 int
 main (int argc, char **argv) 
 {
-	GladeXML     *dialog;
 	gchar        *mime_type;
 	GtkTreeModel *model;
 	MimeTypeInfo *info = NULL;
 	GObject      *mime_dialog;
 
-	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-	bind_textdomain_codeset (PACKAGE, "UTF-8");
-	textdomain (PACKAGE);
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
 
 	if (argc >= 1)
 		mime_type = g_strdup (argv[1]);
@@ -275,10 +281,7 @@ main (int argc, char **argv)
 	gnome_program_init ("gnome-file-types-properties", VERSION, LIBGNOMEUI_MODULE, argc, argv, NULL);
 
 	if (mime_type == NULL) {
-		dialog = create_dialog ();
-
-		g_signal_connect (G_OBJECT (WID ("main_apply_button")), "clicked", (GCallback) apply_cb, NULL);
-		g_signal_connect (G_OBJECT (WID ("main_close_button")), "clicked", (GCallback) gtk_main_quit, NULL);
+		GladeXML *dialog = create_dialog ();
 		gtk_widget_show_all (WID ("main_dialog"));
 	} else {
 		model = GTK_TREE_MODEL (mime_types_model_new (FALSE));
