@@ -43,26 +43,24 @@ apply_settings (void)
 	GConfClient *client;
 
 	gboolean repeat, click;
-	int rate, delay, volume;
-	int bell_volume, bell_pitch, bell_duration;
+	int rate, delay;
+	int click_volume, bell_volume, bell_pitch, bell_duration;
 
-#ifdef HAVE_X11_EXTENSIONS_XF86MISC_H
-	XF86MiscKbdSettings kbdsettings;
-#endif
 	XKeyboardControl kbdcontrol;
         int event_base_return, error_base_return;
 
 	client = gconf_client_get_default ();
 
-	repeat        = gconf_client_get_bool (client, "/gnome/desktop/peripherals/keyboard/repeat",        NULL);
-	click         = gconf_client_get_bool (client, "/gnome/desktop/peripherals/keyboard/click",         NULL);
-	rate          = gconf_client_get_int  (client, "/gnome/desktop/peripherals/keyboard/rate",          NULL);
-	delay         = gconf_client_get_int  (client, "/gnome/desktop/peripherals/keyboard/delay",         NULL);
-	volume        = gconf_client_get_int  (client, "/gnome/desktop/peripherals/keyboard/click_volume",  NULL);
-	bell_volume   = gconf_client_get_int  (client, "/gnome/desktop/peripherals/keyboard/bell_volume",   NULL);
-	bell_pitch    = gconf_client_get_int  (client, "/gnome/desktop/peripherals/keyboard/bell_pitch",    NULL);
-	bell_duration = gconf_client_get_int  (client, "/gnome/desktop/peripherals/keyboard/bell_duration", NULL);
+	repeat        = gconf_client_get_bool  (client, "/desktop/gnome/peripherals/keyboard/repeat",        NULL);
+	click         = gconf_client_get_bool  (client, "/desktop/gnome/peripherals/keyboard/click",         NULL);
+	rate          = gconf_client_get_int   (client, "/desktop/gnome/peripherals/keyboard/rate",          NULL);
+	delay         = gconf_client_get_int   (client, "/desktop/gnome/peripherals/keyboard/delay",         NULL);
+	click_volume  = gconf_client_get_int   (client, "/desktop/gnome/peripherals/keyboard/click_volume",  NULL);
+	bell_volume   = gconf_client_get_int   (client, "/desktop/gnome/peripherals/keyboard/bell_volume",   NULL);
+	bell_pitch    = gconf_client_get_int   (client, "/desktop/gnome/peripherals/keyboard/bell_pitch",    NULL);
+	bell_duration = gconf_client_get_int   (client, "/desktop/gnome/peripherals/keyboard/bell_duration", NULL);
 
+	gdk_error_trap_push ();
         if (repeat) {
 		XAutoRepeatOn (GDK_DISPLAY ());
 #ifdef HAVE_X11_EXTENSIONS_XF86MISC_H
@@ -70,10 +68,13 @@ apply_settings (void)
 					    &event_base_return,
 					    &error_base_return) == True)
 		{
-			kbdsettings.type = 0;
+			/* load the current settings */
+			XF86MiscKbdSettings kbdsettings;
+                        XF86MiscGetKbdSettings (GDK_DISPLAY (), &kbdsettings);
+
+			/* assign the new values */
                         kbdsettings.rate = rate;
                         kbdsettings.delay = delay;
-			kbdsettings.servnumlock = False;
                         XF86MiscSetKbdSettings (GDK_DISPLAY (), &kbdsettings);
                 } else {
                         XAutoRepeatOff (GDK_DISPLAY ());
@@ -83,13 +84,20 @@ apply_settings (void)
 		XAutoRepeatOff (GDK_DISPLAY ());
 	}
 
-	kbdcontrol.key_click_percent = 
-		click ? volume : 0;
+	/* as percentage from 0..100 inclusive */
+	if (click_volume < 0)
+		click_volume = 0;
+	else if (click_volume > 100)
+		click_volume = 100;
+	kbdcontrol.key_click_percent = click ? click_volume : 0;
 	kbdcontrol.bell_percent = bell_volume;
 	kbdcontrol.bell_pitch = bell_pitch;
 	kbdcontrol.bell_duration = bell_duration;
 	XChangeKeyboardControl (GDK_DISPLAY (), KBKeyClickPercent, 
 				&kbdcontrol);
+
+	XSync (GDK_DISPLAY (), FALSE);
+	gdk_error_trap_pop ();
 }
 
 
